/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2014 Esko Stonecube Ltd. All rights reserved.
 */

define([
    "underscore",
    "backbone",
    "./HttpRequestHandler",
    "./LogMessage",
    "./URLUtils"
], function (
    _,
    bb,
    WV3DHttpRequestHandler,
    LogMessage,
    URLUtils
)
{

    "use strict";

    /**
     * @alias module:WebViewer3D
     * @class WV3DHttpRequestManager
     */
    return bb.Model.extend({

        initialize: function (__callback, __errorCallback, __urlPreprocessCallback)
        {
            LogMessage.Log("Creating WV3DHttpRequestManager Object");
            this.set("_callback", __callback);
            this.set("_errorCallback", __errorCallback);
            if (typeof __urlPreprocessCallback === "function")
                this.set("_urlPreprocessCallback", __urlPreprocessCallback);
            else
                this.set("_urlPreprocessCallback", function (__url, __queryCallback)
                {
                    __queryCallback(__url);
                });
        },

        /**
         * IMPROVEME: Make this function more general (so that it works on general URL's).
         *            Then it can probably be moved to URLUtils (idem for GetRealURLWithAppendedRef).
         *            Note that this class becomes more or less obsolete then, since it then just forwards the LoadURL call to the WV3DHttpRequestHandler.
         * Retrieve the real URL of a file, removing the json file prefix
         *
         * @param {String} __sceneURL URL to the scene data
         * @param __parentSceneURL
         * @param __idx_hash
         */
        GetRealURL: function (__sceneURL, __parentSceneURL, __idx_hash)
        {
            //Remove everything that might be located after an eventual # symbol
            //since it is useless for html requests (unless the server itself can understand it)
            let idx_hash;
            if (__idx_hash === undefined)
            {
                idx_hash = __sceneURL.lastIndexOf("#");
            }
            else
            {
                idx_hash = __idx_hash;
            }
            let realSceneURL;
            if (idx_hash === -1)
            {
                realSceneURL = __sceneURL;
            }
            else
            {
                realSceneURL = __sceneURL.substring(0, __sceneURL.indexOf("#"));
            }

            //Get the real URL by removing the base path
            if (URLUtils.IsRelativeURL(realSceneURL) && __parentSceneURL)
            {
                const __parentScenePath = __parentSceneURL.substring(0, __parentSceneURL.lastIndexOf("/") + 1);
                return __parentScenePath.concat(realSceneURL);
            }
            else if (URLUtils.IsAbsoluteURL(__sceneURL))
            {
                return realSceneURL;
            }
            else
            {
                throw "Not a URL";
            }
        },

        /**
         * Retrieve the real URL of a file, removing the json file prefix
         *
         * @param {String} __sceneURL URL to the scene data
         * @param __parentSceneURL
         */
        GetRealURLWithAppendedRef: function (__sceneURL, __parentSceneURL)
        {
            //Remove everything that might be located after an ventual # symbol
            //since it is useless for html requests (unless the server itself can understand it)
            const idx_hash = __sceneURL.lastIndexOf("#");
            let appendedRef = "";
            if (idx_hash !== -1)
            {
                appendedRef = __sceneURL.substring(__sceneURL.indexOf("#"));
            }
            return this.GetRealURL(__sceneURL, __parentSceneURL, idx_hash) + appendedRef;
        },

        /**
         * Load a scene from a URL resource
         *
         * @param {String} __sceneURL URL to the scene data
         */
        LoadURL: function (__sceneURL)
        {
            //Create a proper http request handler that can have its own attributes
            //Give to that new request the callback parameters inherited
            new WV3DHttpRequestHandler(__sceneURL, this.get("_callback"), this.get("_errorCallback"), this.get("_urlPreprocessCallback"));
        }

    });
});
