import React, { useEffect, useRef, useState } from "react";
import ReactDOM from "react-dom";
import PropTypes from "prop-types";
import BootstrapModal from "./Modal";

const Modal = React.memo(function Modal({ body, footer, title, allowClose = false }) {
	const [contentElement, setContent] = useState();
	const [footerElement, setFooter] = useState();
	const modalRef = useRef();
	const isDestroyed = useRef(false);
	const isFooterPresent = !!footer;

	useEffect(() => {
		const newContent = document.createElement("div");
		setContent(newContent);
		modalRef.current = new BootstrapModal(newContent, {
			closeOnBackdropClick: false,
			allowClose: allowClose,
			animate: false // needed, otherwise modal backdrop is not properly destroyed
		});
		modalRef.current.open();

		return () => {
			isDestroyed.current = true;
			modalRef.current.close();
			modalRef.current.destroy();
		};
	}, []);

	useEffect(() => {
		if (isFooterPresent && !isDestroyed.current) {
			const newFooter = document.createElement("div");
			setFooter(newFooter);
			modalRef.current.setFooter(newFooter);
		}
	}, [isFooterPresent]);

	useEffect(() => {
		if (!isDestroyed.current) {
			modalRef.current.setTitle(title);
		}
	}, [title]);

	return (
		<>
			{contentElement && ReactDOM.createPortal(body, contentElement)}
			{footerElement && ReactDOM.createPortal(footer, footerElement)}
		</>
	);
});

Modal.propTypes = {
	body: PropTypes.element,
	footer: PropTypes.element,
	title: PropTypes.string
};

export default Modal;
