/**
 * Handle mousewheel events
 */
var Mousewheel = new function() {
	var listeners = [];

	this.enabled = true;

	window.addEventListener(
		"wheel",
		event => {
			if (this.enabled) {
				var delta = Math.max(-1, Math.min(1, event.deltaY));
				/**
				 * If delta is nonzero, handle it. Basically, delta is now
				 * positive if wheel was scrolled up, and negative, if wheel
				 * was scrolled down.
				 */
				if (delta) {
					listeners.forEach(callback => callback(event, delta * -1));
				}
			}
		},
		{ passive: false } // Explicitly set passive to false, otherwise event handlers cannot call preventDefault()
	);

	this.addListener = function(callback) {
		if (listeners.indexOf(callback) < 0) {
			listeners.push(callback);
		}
	};

	this.removeListener = function(callback) {
		var index = listeners.indexOf(callback);
		if (index > -1) {
			listeners.splice(index, 1);
		}
	};
}();

export default Mousewheel;
