import { SelectionModel } from "./TextSelectionLayerModel";
import { textFromSelection } from "@esko/web-viewer/svg/TextUtils";
import _ from "underscore";

/**
 * Model used for searching text. Also handles the 'select all' functionality
 */
export default class extends SelectionModel {
	constructor(f, channelName, textModel) {
		super(f, channelName);
		this._textSelectionModel = textModel;

		this._hits = [];
		this._query = "";
		this._i = 0;
		textModel.on("hasText.changed", () => this.setQuery(""));
	}

	/**
	 * sets the query to search on inside the page
	 * @param {String} query
	 */
	setQuery(query, caseSensitive = false) {
		if (this._query !== query) {
			this._allSelected = false;
			this._i = 0;
			if (!query) {
				this._query = "";
				this._hits = [];
			} else {
				this._query = query;
				this._hits = this._textSelectionModel.findText(this._query, caseSensitive);
			}
			this._emit("changed.query");
		}
	}
	getQuery() {
		return this._query;
	}
	getHits() {
		return this._hits.map((hit, i) => {
			if (this._i === i) {
				return hit.map(hit => {
					hit = Object.create(hit);
					hit.color = "rgba(226, 152, 29, 0.7)";
					hit.lineIndex = hit.lineIndex + "-highlight";
					return hit;
				});
			}
			return hit;
		});
	}
	/**
	 * @override
	 */
	getSelection() {
		return this._allSelected ? this._textSelectionModel.getCharacters() : _.flatten(this.getHits());
	}
	/**
	 * @override
	 */

	getHighlight() {
		return this._allSelected ? this._textSelectionModel.getCharacters() : this.getHits()[this.getHighlightedIndex()];
	}
	/**
	 * @override
	 */
	getContent() {
		var selection = this.getHighlight();
		return textFromSelection(selection);
	}
	getHighlightedIndex() {
		return this._i;
	}
	hitCount() {
		return this._hits.length;
	}
	_d(dx) {
		this._allSelected = false;
		const hitCount = this.hitCount();
		this._i = (this._i + dx + hitCount) % this.hitCount();
		this._emit("changed.highlight");
	}
	next() {
		const hitCount = this.hitCount();
		if (hitCount > 1) {
			this._d(1);
		}
	}
	previous() {
		if (this.hitCount() > 1) {
			this._d(-1);
		}
	}
	selectAll() {
		this._allSelected = true;
		this._emit("changed.value");
	}
}
