import ToolbarComponent from "@esko/toolbar/ToolbarComponent";
import $ from "jquery";
import Mousetrap from "@esko/mousetrap";
import CoreConstants from "@esko/web-viewer/core/CoreConstants";
import _ from "underscore";
import Tooltip from "@esko/tooltip/Tooltip";

export default class extends ToolbarComponent {
	constructor(f, textSearchModel, textSelectionModel) {
		super("Text Search Toolbar");

		const TEXTSEARCH = "viewerApp-text-search";
		const TOOLBAR = `${TEXTSEARCH}-toolbar`;
		const INPUT_CLASS = `${TEXTSEARCH}-input`;

		const EXIT = `${TEXTSEARCH}-exit`;

		const HITCOUNT = `${TEXTSEARCH}-hitCount`;
		const NOHITS = `${HITCOUNT}-none`;

		const STEPTHROUGH = `${TEXTSEARCH}-stepThrough`;
		const STEPTHROUGH_TOGGLE = `${STEPTHROUGH}-toggle`;
		const STEPTHROUGH_NEXT = `${STEPTHROUGH}-next`;
		const STEPTHROUGH_PREVIOUS = `${STEPTHROUGH}-previous`;

		const ZOOM = `${TEXTSEARCH}-zoom`;

		const SELECT_ALL = `${TEXTSEARCH}-selectAll`;

		var html = `<div class="btn-toolbar ${TOOLBAR}">
				<input class="${INPUT_CLASS}" type="text"></input>
				<span class="${HITCOUNT}"></span>
				<div class="${STEPTHROUGH}">
					<span class="btn btn-xs ${STEPTHROUGH_NEXT} ${STEPTHROUGH_TOGGLE}"><i class="icon icon-sm icon-WCR_Chevron-Down12" /></span>
					<span class="btn btn-xs ${STEPTHROUGH_PREVIOUS} ${STEPTHROUGH_TOGGLE}"><i class="icon icon-sm icon-WCR_Chevron-Up12" /></span>
				</div>
				<div class="btn btn-xs ${ZOOM}"><i class="icon icon-sm icon-WCR_Actions-Search12"/></div>
				<div class="btn btn-xs ${SELECT_ALL}"><i class="icon icon-sm icon-WCR_Actions-SelectAll12"/></div>
				<div class="btn btn-xs ${EXIT}"><i class="icon icon-sm icon-WCR_Close12"/></div>
			</div>
			`;

		this.element = $(html);

		// add tooltips
		f.loadBundle("viewer").then(() => {
			[STEPTHROUGH_NEXT, STEPTHROUGH_PREVIOUS, ZOOM, SELECT_ALL].forEach(c => {
				// use c as tooltip key
				new Tooltip(null, this.element.find("." + c), f.getContext(), f.getContext(), f.isMobile(), f.t(c));
			});
		});
		var updateEl = () => {
			this.element.find(`.${INPUT_CLASS}`).val(textSearchModel.getQuery());
			let hitCount = this.element.find(`.${HITCOUNT}`);
			hitCount.text(`${textSearchModel.getHighlightedIndex() + 1}/${textSearchModel.hitCount()}`);
			hitCount.toggleClass(NOHITS, textSearchModel.hitCount() === 0);
			this.element.find(`.${STEPTHROUGH_TOGGLE}`).toggleClass("disabled", textSearchModel.hitCount() <= 1);
			this.element.find(`.${ZOOM}`).toggleClass("disabled", textSearchModel.hitCount() === 0);
		};

		var zoomToHighlight = () => {
			var hit = textSearchModel.getHits()[textSearchModel.getHighlightedIndex()];
			if (hit) {
				f.emit("viewport.fitInRect", {
					rect: hit.reduce((mem, char) => mem.calculateArea(char.rect), hit.pop().rect),
					animated: true,
					constrain: true,
					maxZoomLevel: -1
				});
			}
		};
		var close = () => f.emit("toolManager.deactivateTool", CoreConstants.TOOLS.TEXTSELECTION);

		textSearchModel.on("changed.*", updateEl);
		updateEl();

		this.element.find(`.${INPUT_CLASS}`).on(
			"input",
			_.throttle(function() {
				textSearchModel.setQuery(this.value);
			}, 30)
		);
		this.element.find(`.${INPUT_CLASS}`).on("keyup", e => {
			if (e.key === "Escape" || e.key === "Esc") {
				close();
			} else if (e.key === "Enter") {
				if (textSearchModel.hitCount() > 1) {
					textSearchModel.next();
				} else if (textSearchModel.hitCount() === 1) {
					zoomToHighlight();
				}
			}
		});

		this.element.on("click", `.${SELECT_ALL}`, () => textSearchModel.selectAll());
		this.element.on("click", `.${STEPTHROUGH_NEXT}`, () => textSearchModel.next());
		this.element.on("click", `.${STEPTHROUGH_PREVIOUS}`, () => textSearchModel.previous());
		this.element.on("click", `.${EXIT}`, () => this.hide());
		this.element.on("click", `.${ZOOM}`, zoomToHighlight);

		textSearchModel.on("changed.highlight", zoomToHighlight);

		const showSearchField = e => {
			if (textSelectionModel.hasText()) {
				f.emit("toolManager.activateTool", CoreConstants.TOOLS.TEXTSELECTION);
				e.preventDefault();
			}
		};

		f.on(`tool.${CoreConstants.TOOLS.TEXTSELECTION}.activated`, () => {
			this.show();
			this.element.find(`.${INPUT_CLASS}`).focus();
		});
		Mousetrap.bind("ctrl+f", showSearchField, "keypress");
	}
}
